#!/usr/bin/env python
"""
params.pxd: Cython functionality for mlpack::util::Params.

This file provides the wrapper to the Params class, along with some of
its methods that are:
Get() - Used to "get" a reference to a parameter with the given name.
Has() - Used to know if a parameter with a given name exists in the program.
SetPassed() - Used to set a parameter as "passed".
CheckInputMatrices() - Sanity check for matrics to know if a matrix has NULL
                       or NaN values.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython
from libcpp.string cimport string
from libcpp cimport bool

cdef extern from "<mlpack/core/util/params.hpp>" namespace "mlpack::util" nogil:
  cdef cppclass Params:
    Params() nogil
    (T&) Get[T](string) nogil except +
    bool Has(string) nogil except +
    void SetPassed(string) nogil except +
    void CheckInputMatrices() nogil except +



#!/usr/bin/env python
#
# Copy all built artifacts from build/lib.*/* to mlpack/.
#
# mlpack is free software; you may redistribute it and/or modify it under the
# terms of the 3-clause BSD license.  You should have received a copy of the
# 3-clause BSD license along with mlpack.  If not, see
# http://www.opensource.org/licenses/BSD-3-Clause for more information.
import sysconfig
import shutil
import os
import glob

# Match any lib.$platform*/mlpack/ directory.
directory = glob.glob('build/lib.' + sysconfig.get_platform() + '*/mlpack/')[0]
directory = directory.replace('\\', '/')

# Now copy all the files from the directory to the desired location.
for f in os.listdir(directory):
  shutil.copy(directory + f, 'mlpack/' + f)



#!/usr/bin/env python
#
# Print the generated installation directory that packages will be installed to,
# so that we can add it to the Python path from CMake.
#
# The argument to this should be the installation prefix.
#
# mlpack is free software; you may redistribute it and/or modify it under the
# terms of the 3-clause BSD license.  You should have received a copy of the
# 3-clause BSD license along with mlpack.  If not, see
# http://www.opensource.org/licenses/BSD-3-Clause for more information.
import sys

print(sys.argv[1] + \
      '/lib/python' + \
      str(sys.version_info.major) + \
      '.' + \
      str(sys.version_info.minor) + \
      '/site-packages/')



#!/usr/bin/python
"""
serialization.pxd: serialization functions for mlpack classes.

This simply makes the utility serialization functions from serialization.hpp
available from Python.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython

from libcpp.string cimport string

cdef extern from "serialization.hpp" namespace "mlpack::bindings::python" nogil:
  string SerializeOut[T](T* t, string name) nogil
  void SerializeIn[T](T* t, string str, string name) nogil
  string SerializeOutJSON[T](T* t, string name) nogil
  void SerializeInJSON[T](T* t, string str, string name) nogil
  



#!/usr/bin/env python
"""
preprocess_json_params.py: utility functions for json paramter preprocessing
                           (see set_cpp_param() and get_cpp_param() methods
                            in print_class_defn.hpp)

The "process_params_out" and "process_params_in" utilities are used to handle
interconversion between the output json from cereal and python dictionary.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
import numpy as np
import json
import pprint
from copy import deepcopy
from collections import OrderedDict

def process_params_out(model, params, return_str=False):
  '''
  This method processes the parameters obtained from the model.

  params:
  1) model - the model to process params.
  2) params - json parameters of the model (which we get through cereal).
  3) return_str (bool) - if True then a pretty string version of the 
                         params is returned.
  '''
  # for pretty printing.
  pp = pprint.PrettyPrinter()

  # value_resolver defined later.
  params_dic = json.loads(params, object_pairs_hook=value_resolver)

  # remove "cereal_class_version".
  # this stores the cereal_class_version value for all deleted pairs.
  cereal_class_version = []
  ref_path = []
  # 'full_paths' will store the complete path in the dictionary for all
  # occurrences. This will be used during the reversed process, to insert
  # 'cereal_class_version' at the correct places to avoid any errors.
  full_paths = []

  scrub(params_dic, "cereal_class_version", cereal_class_version, full_paths,
      ref_path)

  # storing 'cereal_class_version' occurrences paths and values.
  model.scrubbed_params["cereal_class_version"] = {
      "values": cereal_class_version,
      "full_paths": full_paths,
  }

  # convert armadillo dictionary to numpy array.
  arma_to_np(params_dic)

  if return_str:
    return params_dic, pp.pformat(params_dic)
  else:
    return params_dic

def process_params_in(model, params_dic):
  """
  This function takes in a model and the parameters dictionary,
  and returns a string that can be ingested back into the model.
  """
  # deepcopy to prevent changes to the user dictionary.
  params_dic_copy = deepcopy(params_dic)

  # convert numpy to armadillo.
  np_to_arma(params_dic_copy)

  # inserting scrubbed parameters back into dictionary.
  for param_name, details in model.scrubbed_params.items():
    for val, path in zip(details["values"], details["full_paths"]):
      insert_in_dic(params_dic_copy, path, param_name, val)

  # dumping to string. restore_value defined later.
  params_str = json.dumps(params_dic_copy, cls=restore_value)
  return params_str

def np_to_arma(obj):
  """
  This function replaces a numpy array to json representation
  of armadillo vector. This is reverse of "arma_to_np(obj)".
  """
  if isinstance(obj, OrderedDict):
    for key in obj.keys():
      """
      Checking if this is a numpy array.
      """
      if isinstance(obj[key], np.ndarray):
        # n_rows, n_cols have to be strings.
        n_rows, n_cols = obj[key].shape

        dic = OrderedDict()

        dic["n_rows"] = str(n_cols) # implicit transpose
        dic["n_cols"] = str(n_rows) # implicit transpose

        if n_cols != 1 and n_rows != 1:
          dic["vec_state"] = "0"
        elif n_rows == 1:
          dic["vec_state"] = "1"
        elif n_cols == 1:
          dic["vec_state"] = "2"

        elems = obj[key].flatten()
        dic["elem"] = list(elems)
        obj[key] = dic
      else:
        np_to_arma(obj[key])
  elif isinstance(obj, list):
    for i in range(len(obj)):
      np_to_arma(obj[i])
  else:
    # we cannot recurse further if we do not have a
    # dictionary or list object, so just pass.
    pass

def arma_to_np(obj):
  """
  This function replaces the JSON representation of armadillo vector to
  numpy array in the given dictionary.
  """
  if isinstance(obj, OrderedDict):
    for key in obj.keys():
      if isinstance(obj[key], OrderedDict):
        # if "vec_state" is present in dictionary, then it must be an Armadillo
        # vector.
        if "vec_state" in obj[key].keys():
          n_rows = int(obj[key]["n_rows"])
          n_cols = int(obj[key]["n_cols"])

          # Perform an implicit transpose, if there are any elements.
          if n_rows > 0 and n_cols > 0:
            obj[key] = np.array(obj[key]["elem"]).reshape(n_cols,
                n_rows).astype(type(obj[key]["elem"][0]))
          else:
            obj[key] = np.zeros((n_rows, n_cols))
        else:
          arma_to_np(obj[key])
      else:
        arma_to_np(obj[key])
  elif isinstance(obj, list):
    for i in range(len(obj)):
      arma_to_np(obj[i])
  else:
    # we cannot recurse further if we do not have a
    # dictionary or list object, so just pass.
    pass

def scrub(obj, bad_key, values, full_paths, ref_path):
  """
  This function removes a certain key-value pair from the
  given dictionary.
  params:
  1) obj (dict) - dictionary to traverse.
  2) bad_key (str) - key to remove.
  3) values (list) - list of values of all occurrences of bad_key
                    (this will be used to insert bad_key back into dictionary).
  4) full_paths (list) - this is a list that contains full path to all
                         occurrences of bad_key (used to insert bad_key back 
                         into dictionary).
  5) ref_path (list) - this for keeping track of the current path in the
                       dictionary.
  """
  if isinstance(obj, OrderedDict):
    for key in list(obj.keys()):
      ref_path.append(key)
      if key == bad_key:
        ref_path.pop()
        ref_path_copy = deepcopy(ref_path)
        full_paths.append(ref_path_copy)
        values.append(obj[key])
        del obj[key]
      else:
        scrub(obj[key], bad_key, values, full_paths, ref_path)
    if ref_path != []:
      ref_path.pop()  
  elif isinstance(obj, list):
    for i in range(len(obj)):
      ref_path.append(f"listidx_{i}")
      scrub(obj[i], bad_key, values, full_paths, ref_path)
    if ref_path != []:
      ref_path.pop()  
  else:
    ref_path.pop()
    pass

def value_resolver(pairs):
  '''
  This function converts multiple "elem" occurences to a list when
  used with json.loads().
  Eg:
  str({
    vec_state: 1,
    n_rows: 2,
    n_cols: 1,
    elem: 1,
    elem: 2
  })

  will be converted to

  dict({
    vec_state: 1,
    n_rows: 2,
    n_cols: 1,
    elem: [1,2]
  })
  This is done to handle same keys in the json while converting to python
  dictionary.
  '''
  has_elem = False
  for key,val in pairs:
    if key == "elem":
      has_elem = True
      break
  if has_elem:
    val_list = [val for (key,val) in pairs if key == "elem"]
    pairs = [(key,val) for (key,val) in pairs if key != "elem"]
    pairs.append(("elem", val_list))
  return OrderedDict(pairs)

class restore_value(json.JSONEncoder):
  '''
  This is a custom encoder that converts a dictionary to
  correct json format for ingesting in cereal.
  Eg:
  dict({
    vec_state: 1,
    n_rows: 2,
    n_cols: 1,
    elem: [1,2]
  })

  will be converted into

  str({
    vec_state: 1,
    n_rows: 2,
    n_cols: 1,
    elem: 1,
    elem: 2
  })
  while encoding.
  This is used to create a json that can be ingested to cereal.
  '''
  def encode(self, o):
    if isinstance(o, dict):
      if "elem" in o.keys():
        to_return = '{%s' % ', '.join(
            ': '.join((json.encoder.py_encode_basestring(k), self.encode(v)))\
                for k, v in o.items() if k != "elem")
        for val in o["elem"]:
          to_return += ', ' + json.encoder.py_encode_basestring("elem") +\
              f': {val}'
        to_return += "}"
        return to_return
      else:
        to_return = '{%s}' % ', '.join(
            ': '.join((json.encoder.py_encode_basestring(k), self.encode(v)))\
                for k, v in o.items())
        return to_return
    if isinstance(o, list):
      to_return = '[%s]' % ', '.join((self.encode(k) for k in o))
      return to_return
    return super().encode(o)

def insert_in_dic(dic, path, key, val):
  '''
  This function inserts a particluar key-value pair in a dictionray
  after following a particular path.
  '''
  temp = dic[path[0]]
  for idx in range(1, len(path)):
    if "listidx_" in path[idx]:
      temp = temp[int(path[idx].replace("listidx_", ""))]
    else:
      temp = temp[path[idx]]
  temp[key] = val
  # moving key-value pair to the start.
  temp.move_to_end(key, last=False)



#!/usr/bin/env python
"""
timers.pxd: Cython wrapper for Timers.

This file provides a basic wrapper for Timers class, that is used in calling
the main program function.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython

cdef extern from "<mlpack/core/util/timers.hpp>" namespace "mlpack::util" nogil:
  cdef cppclass Timers:
    Timers() nogil



#!/usr/bin/env python
"""
io.pyx: Cython functionality for mlpack::IO and other utilities.

This file imports the Parameters() function from mlpack::IO, plus other utility
functions: SetParam(), SetParamPtr(), SetParamWithInfo(), GetParam(),
GetParamWithInfo(), EnableVerbose(), DisableVerbose(), DisableBacktrace(),
EnableTimers() and ResetTimers().

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython
from libcpp.string cimport string
from libcpp cimport bool
from .params cimport Params

cdef extern from "<mlpack/core/util/io.hpp>" namespace "mlpack" nogil:
  cdef cppclass IO:
    @staticmethod
    Params Parameters(string) nogil except +

cdef extern from "<mlpack/bindings/python/mlpack/io_util.hpp>" \
    namespace "mlpack::util" nogil:
  void SetParam[T](Params, string, T&) nogil except +
  void SetParam[T](Params, string, T&, bool) nogil except +
  void SetParamPtr[T](Params, string, T*, bool) nogil except +
  void SetParamWithInfo[T](Params, string, T&, const bool*) nogil except +
  (T*) GetParamPtr[T](Params, string) nogil except +
  (T&) GetParamWithInfo[T](Params, string) nogil except +
  void EnableVerbose() nogil except +
  void DisableVerbose() nogil except +
  void DisableBacktrace() nogil except +
  void ResetTimers() nogil except +
  void EnableTimers() nogil except +



#!/usr/bin/env python
"""
arma_numpy.pxd: Armadillo/numpy interface functionality.

This file defines a number of functions useful for converting between Armadillo
and numpy objects without actually copying memory.  Because Cython support for
templates is primitive, we can't use templates here and instead overload for
each type we might need.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython
cimport numpy
import numpy

numpy.import_array()

from .arma cimport Mat, Row, Col
from libcpp cimport bool

"""
Convert a numpy ndarray to a matrix.
"""
cdef Mat[double]* numpy_to_mat_d(numpy.ndarray[numpy.double_t, ndim=2] X, \
                                 bool takeOwnership) except +
cdef Mat[size_t]* numpy_to_mat_s(numpy.ndarray[numpy.npy_intp, ndim=2] X, \
                                 bool takeOwnership) except +

"""
Convert an Armadillo object to a numpy ndarray of the given type.
"""
cdef numpy.ndarray[numpy.double_t, ndim=2] mat_to_numpy_d(Mat[double]& X) \
    except +
cdef numpy.ndarray[numpy.npy_intp, ndim=2] mat_to_numpy_s(Mat[size_t]& X) \
    except +

"""
Convert a numpy one-dimensional ndarray to a row of the given type.
"""
cdef Row[double]* numpy_to_row_d(numpy.ndarray[numpy.double_t, ndim=1] X, \
                                 bool takeOwnership) except +
cdef Row[size_t]* numpy_to_row_s(numpy.ndarray[numpy.npy_intp, ndim=1] X, \
                                 bool takeOwnership) except +

"""
Convert an Armadillo row vector to a one-dimensional numpy ndarray of the
given type.
"""
cdef numpy.ndarray[numpy.double_t, ndim=1] row_to_numpy_d(Row[double]& X) \
    except +
cdef numpy.ndarray[numpy.npy_intp, ndim=1] row_to_numpy_s(Row[size_t]& X) \
    except +

"""
Convert a numpy one-dimensional ndarray to a column vector of the given type.
"""
cdef Col[double]* numpy_to_col_d(numpy.ndarray[numpy.double_t, ndim=1] X, \
                                 bool takeOwnership) except +
cdef Col[size_t]* numpy_to_col_s(numpy.ndarray[numpy.npy_intp, ndim=1] X, \
                                 bool takeOwnership) except +

"""
Convert an Armadillo column vector to a one-dimensional numpy ndarray of the
given type.
"""
cdef numpy.ndarray[numpy.double_t, ndim=1] col_to_numpy_d(Col[double]& X) \
    except +
cdef numpy.ndarray[numpy.npy_intp, ndim=1] col_to_numpy_s(Col[size_t]& X) \
    except +



#!/usr/bin/env python
"""
arma.pxd: Armadillo extensions for Cython.

This file defines the Armadillo matrix class and all of the symbols that are
needed for mlpack's Python bindings.  Note that only the necessary symbols are
included here---not the entire Armadillo library.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython

from libcpp cimport bool

# We have to include from mlpack/core.hpp so that everything is included in the
# right order and the Armadillo extensions from mlpack are set up right.
cdef extern from "<mlpack/core.hpp>" namespace "arma" nogil:
  # Import the index type.
  ctypedef int uword
  # Import the half-size index type.
  ctypedef int uhword

  cdef cppclass Mat[T]:
    # Special constructor that uses auxiliary memory.
    Mat(T* aux_mem,
        uword n_rows,
        uword n_cols,
        bool copy_aux_mem,
        bool strict) nogil

    # Constructor that initializes memory.
    Mat(uword n_rows, uword n_cols) nogil

    # Default constructor.
    Mat() nogil

    # Number of rows.
    const uword n_rows
    # Number of columns.
    const uword n_cols
    # Total number of elements.
    const uword n_elem

    # Memory state: preallocated, changeable, etc.
    const uhword mem_state

    # Access the memory pointer directly.
    T* memptr() nogil

  cdef cppclass Col[T]:
    # Special constructor that uses auxiliary memory.
    Col(T* aux_mem,
        uword n_elem,
        bool copy_aux_mem,
        bool strict) nogil

    # Constructor that initializes memory.
    Col(uword n_elem) nogil

    # Default constructor.
    Col() nogil

    # Number of rows (equal to number of elements).
    const uword n_rows
    # Number of columns (always 1).
    const uword n_cols
    # Number of elements (equal to number of rows).
    const uword n_elem

    # Memory state: preallocated, changeable, etc.
    const uword mem_state

    # Access the memory pointer directly.
    T* memptr() nogil

  cdef cppclass Row[T]:
    # Special constructor that uses auxiliary memory.
    Row(T* aux_mem,
        uword n_elem,
        bool copy_aux_mem,
        bool strict) nogil

    # Constructor that initializes memory.
    Row(uword n_elem) nogil

    # Default constructor.
    Row() nogil

    # Number of rows (always 1).
    const uword n_rows
    # Number of columns (equal to number of elements).
    const uword n_cols
    # Number of elements (equal to number of columns).
    const uword n_elem

    # Memory state: preallocated, changeable, etc.
    const uword mem_state

    # Access the memory pointer directly.
    T* memptr() nogil



#!/usr/bin/env python
"""
arma_numpy.pyx: Armadillo/numpy interface functionality.

This file defines a number of functions useful for converting between Armadillo
and numpy objects without actually copying memory.  Note that if a numpy matrix
is converted to an Armadillo object, then the Armadillo object will "own" the
matrix and free the memory upon destruction (and the numpy object will no longer
"own" the matrix).  Similarly, if an Armadillo object is converted to a numpy
object, then the numpy object will "own" the matrix.

Thus, know that if you convert a matrix type, remember that the resulting type
is what "owns" the allocated memory.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
cimport cython
cimport numpy
import numpy

numpy.import_array()

from .arma cimport Mat, Row, Col
from libcpp cimport bool

import platform
isWin = (platform.system() == "Windows")

cdef extern from "numpy/arrayobject.h":
  void PyArray_ENABLEFLAGS(numpy.ndarray arr, int flags)
  void PyArray_CLEARFLAGS(numpy.ndarray arr, int flags)
  void* PyArray_DATA(numpy.ndarray arr)
  numpy.npy_intp* PyArray_SHAPE(numpy.ndarray arr)
  int PyArray_FLAGS(numpy.ndarray arr)

cdef extern from "<mlpack/bindings/python/mlpack/arma_util.hpp>":
  void SetMemState[T](T& m, int state)
  size_t GetMemState[T](T& m)
  double* GetMemory(Mat[double]& m)
  double* GetMemory(Col[double]& m)
  double* GetMemory(Row[double]& m)
  size_t* GetMemory(Mat[size_t]& m)
  size_t* GetMemory(Col[size_t]& m)
  size_t* GetMemory(Row[size_t]& m)

cdef Mat[double]* numpy_to_mat_d(numpy.ndarray[numpy.double_t, ndim=2] X, \
                                 bool takeOwnership) except +:
  """
  Convert a numpy ndarray to a matrix.  The memory will still be owned by numpy.
  """
  cdef int flags = PyArray_FLAGS(X)
  if not (flags & numpy.NPY_ARRAY_C_CONTIGUOUS) or \
    (not (flags & numpy.NPY_ARRAY_OWNDATA) and not isWin):
    # If needed, make a copy where we own the memory.
    X = X.copy(order="C")
    takeOwnership = True

  cdef Mat[double]* m = new Mat[double](<double*> PyArray_DATA(X),
      PyArray_SHAPE(X)[1], PyArray_SHAPE(X)[0], isWin, False)

  # Take ownership of the memory, if we need to and we are not on Windows.
  if takeOwnership and not isWin:
    PyArray_CLEARFLAGS(X, numpy.NPY_ARRAY_OWNDATA)
    SetMemState[Mat[double]](m[0], 0)

  return m

cdef Mat[size_t]* numpy_to_mat_s(numpy.ndarray[numpy.npy_intp, ndim=2] X, \
                                 bool takeOwnership) except +:
  """
  Convert a numpy ndarray to a matrix.  The memory will still be owned by numpy.
  """
  cdef int flags = PyArray_FLAGS(X)
  if not (flags & numpy.NPY_ARRAY_C_CONTIGUOUS) or \
    (not (flags & numpy.NPY_ARRAY_OWNDATA) and not isWin):
    # If needed, make a copy where we own the memory, except on Windows where
    # we never copy.
    X = X.copy(order="C")
    takeOwnership = True

  cdef Mat[size_t]* m = new Mat[size_t](<size_t*> PyArray_DATA(X),
      PyArray_SHAPE(X)[1], PyArray_SHAPE(X)[0], isWin, False)

  # Take ownership of the memory, if we need to.
  if takeOwnership and not isWin:
    PyArray_CLEARFLAGS(X, numpy.NPY_ARRAY_OWNDATA)
    SetMemState[Mat[size_t]](m[0], 0)

  return m

cdef numpy.ndarray[numpy.double_t, ndim=2] mat_to_numpy_d(Mat[double]& X) \
    except +:
  """
  Convert an Armadillo object to a numpy ndarray.
  """
  # Extract dimensions.
  cdef numpy.npy_intp dims[2]
  dims[0] = <numpy.npy_intp> X.n_cols
  dims[1] = <numpy.npy_intp> X.n_rows
  cdef numpy.ndarray[numpy.double_t, ndim=2] output = \
      numpy.PyArray_SimpleNewFromData(2, &dims[0], numpy.NPY_DOUBLE, GetMemory(X))
  if isWin:
    output = output.copy(order="C")

  # Transfer memory ownership, if needed.
  if GetMemState[Mat[double]](X) == 0 and not isWin:
    SetMemState[Mat[double]](X, 1)
    PyArray_ENABLEFLAGS(output, numpy.NPY_ARRAY_OWNDATA)

  return output

cdef numpy.ndarray[numpy.npy_intp, ndim=2] mat_to_numpy_s(Mat[size_t]& X) \
    except +:
  """
  Convert an Armadillo object to a numpy ndarray.
  """
  # Extract dimensions.
  cdef numpy.npy_intp dims[2]
  dims[0] = <numpy.npy_intp> X.n_cols
  dims[1] = <numpy.npy_intp> X.n_rows
  cdef numpy.ndarray[numpy.npy_intp, ndim=2] output = \
      numpy.PyArray_SimpleNewFromData(2, &dims[0], numpy.NPY_INTP, GetMemory(X))
  if isWin:
    output = output.copy(order="C")

  # Transfer memory ownership, if needed.
  if GetMemState[Mat[size_t]](X) == 0 and not isWin:
    SetMemState[Mat[size_t]](X, 1)
    PyArray_ENABLEFLAGS(output, numpy.NPY_ARRAY_OWNDATA)

  return output

cdef Row[double]* numpy_to_row_d(numpy.ndarray[numpy.double_t, ndim=1] X, \
                                 bool takeOwnership) except +:
  """
  Convert a numpy one-dimensional ndarray to a row.  The memory will still be
  owned by numpy.
  """
  cdef int flags = PyArray_FLAGS(X)
  if not (flags & numpy.NPY_ARRAY_C_CONTIGUOUS) or \
    (not (flags & numpy.NPY_ARRAY_OWNDATA) and not isWin):
    # If needed, make a copy where we own the memory, except on Windows where
    # we never copy.
    X = X.copy(order="C")
    takeOwnership = True

  cdef Row[double]* m = new Row[double](<double*> PyArray_DATA(X),
    PyArray_SHAPE(X)[0], isWin, False)

  # Transfer memory ownership, if needed.
  if takeOwnership and not isWin:
    PyArray_CLEARFLAGS(X, numpy.NPY_ARRAY_OWNDATA)
    SetMemState[Row[double]](m[0], 0)

  return m

cdef Row[size_t]* numpy_to_row_s(numpy.ndarray[numpy.npy_intp, ndim=1] X, \
                                 bool takeOwnership) except +:
  """
  Convert a numpy one-dimensional ndarray to a row.  The memory will still be
  owned by numpy.
  """
  cdef int flags = PyArray_FLAGS(X)
  if not (flags & numpy.NPY_ARRAY_C_CONTIGUOUS) or \
    (not (flags & numpy.NPY_ARRAY_OWNDATA) and not isWin):
    # If needed, make a copy where we own the memory, except on Windows where
    # we never copy.
    X = X.copy(order="C")
    takeOwnership = True

  cdef Row[size_t]* m = new Row[size_t](<size_t*> PyArray_DATA(X),
      PyArray_SHAPE(X)[0], isWin, False)

  # Transfer memory ownership, if needed.
  if takeOwnership and not isWin:
    PyArray_CLEARFLAGS(X, numpy.NPY_ARRAY_OWNDATA)
    SetMemState[Row[size_t]](m[0], 0)

  return m

cdef numpy.ndarray[numpy.double_t, ndim=1] row_to_numpy_d(Row[double]& X) \
    except +:
  """
  Convert an Armadillo row vector to a one-dimensional numpy ndarray.
  """
  # Extract dimensions.
  cdef numpy.npy_intp dim = <numpy.npy_intp> X.n_elem
  cdef numpy.ndarray[numpy.double_t, ndim=1] output = \
      numpy.PyArray_SimpleNewFromData(1, &dim, numpy.NPY_DOUBLE, GetMemory(X))
  if isWin:
    output = output.copy(order="C")

  # Transfer memory ownership, if needed.
  if GetMemState[Row[double]](X) == 0 and not isWin:
    SetMemState[Row[double]](X, 1)
    PyArray_ENABLEFLAGS(output, numpy.NPY_ARRAY_OWNDATA)

  return output

cdef numpy.ndarray[numpy.npy_intp, ndim=1] row_to_numpy_s(Row[size_t]& X) \
    except +:
  """
  Convert an Armadillo row vector to a one-dimensional numpy ndarray.
  """
#  print("called row_to_numpy_s()\n")
  # Extract dimensions.
  cdef numpy.npy_intp dim = <numpy.npy_intp> X.n_elem
  cdef numpy.ndarray[numpy.npy_intp, ndim=1] output = \
      numpy.PyArray_SimpleNewFromData(1, &dim, numpy.NPY_INTP, GetMemory(X))
  if isWin:
    output = output.copy(order="C")

  # Transfer memory ownership, if needed.
  if GetMemState[Row[size_t]](X) == 0 and not isWin:
    SetMemState[Row[size_t]](X, 1)
    PyArray_ENABLEFLAGS(output, numpy.NPY_ARRAY_OWNDATA)

  return output

cdef Col[double]* numpy_to_col_d(numpy.ndarray[numpy.double_t, ndim=1] X, \
                                 bool takeOwnership) except +:
  """
  Convert a numpy one-dimensional ndarray to a column vector.  The memory will
  still be owned by numpy.
  """
  cdef int flags = PyArray_FLAGS(X)
  if not (flags & numpy.NPY_ARRAY_C_CONTIGUOUS) or \
    (not (flags & numpy.NPY_ARRAY_OWNDATA) and not isWin):
    # If needed, make a copy where we own the memory, except on Windows where
    # we never copy.
    X = X.copy(order="C")
    takeOwnership = True

  cdef Col[double]* m = new Col[double](<double*> PyArray_DATA(X),
      PyArray_SHAPE(X)[0], isWin, False)

  # Transfer memory ownership, if needed.
  if takeOwnership and not isWin:
    PyArray_CLEARFLAGS(X, numpy.NPY_ARRAY_OWNDATA)
    SetMemState[Col[double]](m[0], 0)

  return m

cdef Col[size_t]* numpy_to_col_s(numpy.ndarray[numpy.npy_intp, ndim=1] X, \
                                      bool takeOwnership) except +:
  """
  Convert a numpy one-dimensional ndarray to a column vector.  The memory will
  still be owned by numpy.
  """
  cdef int flags = PyArray_FLAGS(X)
  if not (flags & numpy.NPY_ARRAY_C_CONTIGUOUS) or \
    (not (flags & numpy.NPY_ARRAY_OWNDATA) and not isWin):
    # If needed, make a copy where we own the memory, except on Windows where
    # we never copy.
    X = X.copy(order="C")
    takeOwnership = True

  cdef Col[size_t]* m = new Col[size_t](<size_t*> PyArray_DATA(X), 
      PyArray_SHAPE(X)[0], isWin, False)

  # Transfer memory ownership, if needed.
  if takeOwnership and not isWin:
    PyArray_CLEARFLAGS(X, numpy.NPY_ARRAY_OWNDATA)
    SetMemState[Col[size_t]](m[0], 0)

  return m

cdef numpy.ndarray[numpy.double_t, ndim=1] col_to_numpy_d(Col[double]& X) \
    except +:
  """
  Convert an Armadillo column vector to a one-dimensional numpy ndarray.
  """
  # Extract dimension.
  cdef numpy.npy_intp dim = <numpy.npy_intp> X.n_elem
  cdef numpy.ndarray[numpy.double_t, ndim=1] output = \
      numpy.PyArray_SimpleNewFromData(1, &dim, numpy.NPY_DOUBLE, GetMemory(X))
  if isWin:
    output = output.copy(order="C")

  # Transfer memory ownership, if needed.
  if GetMemState[Col[double]](X) == 0 and not isWin:
    SetMemState[Col[double]](X, 1)
    PyArray_ENABLEFLAGS(output, numpy.NPY_ARRAY_OWNDATA)

  return output

cdef numpy.ndarray[numpy.npy_intp, ndim=1] col_to_numpy_s(Col[size_t]& X) \
    except +:
  """
  Convert an Armadillo column vector to a one-dimensional numpy ndarray.
  """
  # Extract dimension.
  cdef numpy.npy_intp dim = <numpy.npy_intp> X.n_elem
  cdef numpy.ndarray[numpy.npy_intp, ndim=1] output = \
      numpy.PyArray_SimpleNewFromData(1, &dim, numpy.NPY_INTP, GetMemory(X))
  if isWin:
    output = output.copy(order="C")

  # Transfer memory ownership, if needed.
  if GetMemState[Col[size_t]](X) == 0 and not isWin:
    SetMemState[Col[size_t]](X, 1)
    PyArray_ENABLEFLAGS(output, numpy.NPY_ARRAY_OWNDATA)

  return output



#/usr/bin/env python
"""
test_dataset_info.py

Test that to_matrix() and to_matrix_with_info() return the correct types.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
import unittest
import pandas as pd
import numpy as np

from mlpack.matrix_utils import to_matrix
from mlpack.matrix_utils import to_matrix_with_info

class TestToMatrix(unittest.TestCase):
  """
  This class defines tests for the to_matrix() and to_matrix_with_info() utility
  functions.
  """

  def testPandasToMatrix(self):
    """
    Test that a simple pandas numeric matrix can be turned into a numpy ndarray.
    """
    d = pd.DataFrame(np.random.randn(100, 4), columns=list('abcd'))

    m, _ = to_matrix(d)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.shape[0], 100)
    self.assertEqual(m.shape[1], 4)
    self.assertEqual(m.dtype, np.dtype(np.double))
    colnames = list('abcd')
    for i in range(m.shape[1]):
      for j in range(m.shape[0]):
        self.assertEqual(m[j, i], d[colnames[i]][j])

  def testPandasIntToMatrix(self):
    """
    Test that a matrix holding ints is properly turned into a double matrix.
    """
    d = pd.DataFrame({'a': range(5)})

    m, _ = to_matrix(d)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.shape[0], 5)
    self.assertEqual(m.shape[1], 1)
    for i in range(5):
      self.assertEqual(m[i], i)

  def testPandasMixedToMatrix(self):
    """
    Test that a matrix with one int and one double feature are transformed
    correctly.
    """
    d = pd.DataFrame({'a': range(50)})
    d['b'] = np.random.randn(50, 1)
    self.assertTrue((d['a'].dtype == np.dtype('int32')) or
                    (d['a'].dtype == np.dtype('int64')))
    self.assertEqual(d['b'].dtype, np.dtype(np.double))

    m, _ = to_matrix(d)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))
    self.assertEqual(m.shape[0], 50)
    self.assertEqual(m.shape[1], 2)
    colNames = list('ab')
    for i in range(2):
      for j in range(50):
        self.assertEqual(d[colNames[i]][j], m[j, i])

  def testArraylikeToMatrix(self):
    """
    Test that if we pass some array, we get back the right thing.  This array
    will be filled with doubles only.
    """
    a = [[0.01, 0.02, 0.03],
         [0.04, 0.05, 0.06],
         [0.07, 0.08, 0.09],
         [0.10, 0.11, 0.12]]

    m, _ = to_matrix(a)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))
    self.assertEqual(m.shape[0], 4)
    self.assertEqual(m.shape[1], 3)

    for i in range(4):
      for j in range(3):
        self.assertEqual(a[i][j], m[i, j])

  def testMultitypeArraylikeToMatrix(self):
    """
    Test that if we pass an array with multiple types, we get back the right
    thing.  The numpy ndarray should be filled with doubles only.
    """
    a = [[0.01, 0.02, 3],
         [0.04, 0.05, 6],
         [0.07, 0.08, 9],
         [0.10, 0.11, 12]]

    m, _ = to_matrix(a)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))
    self.assertEqual(m.shape[0], 4)
    self.assertEqual(m.shape[1], 3)

    for i in range(4):
      for j in range(3):
        self.assertEqual(a[i][j], m[i, j])

  def testNumpyToMatrix(self):
    """
    Make sure we can convert a numpy matrix without copying anything.
    """
    m1 = np.random.randn(100, 5)
    m2, _ = to_matrix(m1)

    self.assertTrue(isinstance(m2, np.ndarray))
    self.assertEqual(m2.dtype, np.dtype(np.double))

    p1 = m1.__array_interface__
    p2 = m2.__array_interface__

    self.assertEqual(p1['data'], p2['data'])

  def testPandasToMatrixNoCategorical(self):
    """
    Make sure that if we pass a Pandas dataframe with no categorical features,
    we get back the matrix we expect.
    """

class TestToMatrixWithInfo(unittest.TestCase):
  """
  This class defines tests for the to_matrix() and to_matrix_with_info() utility
  functions.
  """

  def testPandasToMatrix(self):
    """
    Test that a simple pandas numeric matrix can be turned into a numpy ndarray.
    """
    d = pd.DataFrame(np.random.randn(100, 4), columns=list('abcd'))

    m, _, dims = to_matrix_with_info(d, np.double)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.shape[0], 100)
    self.assertEqual(m.shape[1], 4)
    self.assertEqual(m.dtype, np.dtype(np.double))
    colnames = list('abcd')
    for i in range(m.shape[1]):
      for j in range(m.shape[0]):
        self.assertEqual(m[j, i], d[colnames[i]][j])

    self.assertTrue(dims.shape[0], 4)
    self.assertEqual(dims[0], 0)
    self.assertEqual(dims[1], 0)
    self.assertEqual(dims[2], 0)
    self.assertEqual(dims[3], 0)

  def testPandasIntToMatrix(self):
    """
    Test that a matrix holding ints is properly turned into a double matrix.
    """
    d = pd.DataFrame({'a': range(5)})

    m, _, dims = to_matrix_with_info(d, np.double)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.shape[0], 5)
    self.assertEqual(m.shape[1], 1)
    for i in range(5):
      self.assertEqual(m[i], i)

    self.assertTrue(dims.shape[0], 1)
    self.assertEqual(dims[0], 0)

  def testPandasMixedToMatrix(self):
    """
    Test that a matrix with one int and one double feature are transformed
    correctly.
    """
    d = pd.DataFrame({'a': range(50)})
    d['b'] = np.random.randn(50, 1)
    self.assertTrue((d['a'].dtype == np.dtype('int32')) or
                    (d['a'].dtype == np.dtype('int64')))
    self.assertEqual(d['b'].dtype, np.dtype(np.double))

    m, _, dims = to_matrix_with_info(d, np.double)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))
    self.assertEqual(m.shape[0], 50)
    self.assertEqual(m.shape[1], 2)
    colNames = list('ab')
    for i in range(2):
      for j in range(50):
        self.assertEqual(d[colNames[i]][j], m[j, i])

    self.assertEqual(dims.shape[0], 2)
    self.assertEqual(dims[0], 0)
    self.assertEqual(dims[1], 0)

  def testArraylikeToMatrix(self):
    """
    Test that if we pass some array, we get back the right thing.  This array
    will be filled with doubles only.
    """
    a = [[0.01, 0.02, 0.03],
         [0.04, 0.05, 0.06],
         [0.07, 0.08, 0.09],
         [0.10, 0.11, 0.12]]

    m, _, dims = to_matrix_with_info(a, np.double)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))
    self.assertEqual(m.shape[0], 4)
    self.assertEqual(m.shape[1], 3)

    for i in range(4):
      for j in range(3):
        self.assertEqual(a[i][j], m[i, j])

    self.assertEqual(dims.shape[0], 3)
    self.assertEqual(dims[0], 0)
    self.assertEqual(dims[1], 0)
    self.assertEqual(dims[2], 0)

  def testMultitypeArraylikeToMatrix(self):
    """
    Test that if we pass an array with multiple types, we get back the right
    thing.  The numpy ndarray should be filled with doubles only.
    """
    a = [[0.01, 0.02, 3],
         [0.04, 0.05, 6],
         [0.07, 0.08, 9],
         [0.10, 0.11, 12]]

    m, _, dims = to_matrix_with_info(a, np.double)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))
    self.assertEqual(m.shape[0], 4)
    self.assertEqual(m.shape[1], 3)

    for i in range(4):
      for j in range(3):
        self.assertEqual(a[i][j], m[i, j])

    self.assertEqual(dims.shape[0], 3)
    self.assertEqual(dims[0], 0)
    self.assertEqual(dims[1], 0)
    self.assertEqual(dims[2], 0)

  def testNumpyToMatrix(self):
    """
    Make sure we can convert a numpy matrix without copying anything.
    """
    m1 = np.random.randn(100, 5)
    m2, _, dims = to_matrix_with_info(m1, np.double)

    self.assertTrue(isinstance(m2, np.ndarray))
    self.assertEqual(m2.dtype, np.dtype(np.double))

    p1 = m1.__array_interface__
    p2 = m2.__array_interface__

    self.assertEqual(p1['data'], p2['data'])

    self.assertEqual(dims.shape[0], 5)
    self.assertEqual(dims[0], 0)
    self.assertEqual(dims[1], 0)
    self.assertEqual(dims[2], 0)
    self.assertEqual(dims[3], 0)
    self.assertEqual(dims[4], 0)

  def testCategoricalOnly(self):
    """
    Make sure that we can convert a categorical-only Pandas matrix.
    """
    d = pd.DataFrame({"A": ["a", "b", "c", "a"] })
    d["A"] = d["A"].astype('category') # Convert to categorical.

    m, _, dims = to_matrix_with_info(d, np.double)

    self.assertTrue(isinstance(m, np.ndarray))
    self.assertEqual(m.dtype, np.dtype(np.double))

    self.assertEqual(dims.shape[0], 1)
    self.assertEqual(dims[0], 1)

    self.assertEqual(m.shape[0], 4)
    self.assertEqual(m.shape[1], 1)
    self.assertEqual(m[0], m[3])
    self.assertTrue(m[0] != m[1])
    self.assertTrue(m[1] != m[2])
    self.assertTrue(m[0] != m[2])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
This is an autogenerated file that allows convenient imports of mlpack
functionality.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
import warnings
warnings.filterwarnings("default", category=ImportWarning)



#!/usr/bin/env python
"""
matrix_utils.py: utilities for matrix conversion

This file defines the to_matrix() function, which can be used to convert Pandas
dataframes or other types of array-like objects to numpy ndarrays for use in
mlpack bindings.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
import numpy as np
import pandas as pd
# The CategoricalDtype class has moved multiple times, so this insanity is
# necessary to import the right version.
if int(pd.__version__.split('.')[0]) > 0 or \
   int(pd.__version__.split('.')[1]) >= 20:
  from pandas.api.types import CategoricalDtype
elif int(pd.__version__.split('.')[1]) >= 18:
  from pandas.types.dtypes import CategoricalDtype
elif int(pd.__version__.split('.')[1]) == 17:
  from pandas.core.dtypes import CategoricalDtype
elif int(pd.__version__.split('.')[1]) >= 15:
  from pandas.core.common import CategoricalDtype

# We need a unicode type, but on python3 we don't have it.
try:
  UNICODE_EXISTS = bool(type(unicode))
except NameError:
  unicode = str

# We also need a buffer type.
try:
  BUFFER_EXISTS = bool(type(buffer))
except:
  buffer = memoryview

def to_matrix(x, dtype=np.double, copy=False):
  """
  Given some array-like X, return a numpy ndarray of the same type.
  """
  # Make sure it's array-like at all.
  if not hasattr(x, '__len__') and \
      not hasattr(x, 'shape') and \
      not hasattr(x, '__array__'):
    raise TypeError("given argument is not array-like")

  if (isinstance(x, np.ndarray) and x.dtype == dtype and x.flags.c_contiguous):
    if copy: # Copy the matrix if required.
      return x.copy("C"), True
    else:
      return x, False
  elif (isinstance(x, np.ndarray) and x.dtype == dtype and x.flags.f_contiguous):
    # A copy is always necessary here.
    return x.copy("C"), True
  else:
    if isinstance(x, pd.core.series.Series) or isinstance(x, pd.DataFrame):
      # We can only avoid a copy if the dtype is the same and the copy flag is
      # false.  I'm actually not sure if this is possible, since in everything I
      # have found, Pandas stores with F_CONTIGUOUS not C_CONTIGUOUS.
      y = x.values
      if copy == False and y.dtype == dtype and y.flags.c_contiguous:
        return np.ndarray(y.shape, buffer=x.values, dtype=dtype, order='C'),\
            False
      else:
        # We have to make a copy or change the dtype, so just do this directly.
        return np.array(y, dtype=dtype, order='C', copy=True), True
    else:
      return np.array(x, copy=True, dtype=dtype, order='C'), True


def to_matrix_with_info(x, dtype, copy=False):
  """
  Given some array-like X (which should be either a numpy ndarray or a pandas
  DataFrame), convert it into a numpy matrix of the given dtype.
  """
  # Make sure it's array-like at all.
  if not hasattr(x, '__len__') and \
      not hasattr(x, 'shape') and \
      not hasattr(x, '__array__'):
    raise TypeError("given argument is not array-like")

  if isinstance(x, np.ndarray):
    # It is already an ndarray, so the vector of info is all 0s (all numeric).
    if len(x.shape) < 2:
      d = np.zeros(1, dtype=bool)
    else:
      d = np.zeros([x.shape[1]], dtype=bool)

    # Copy the matrix if needed.
    if copy:
      return (x.copy(order="C"), True, d)
    else:
      return (x, False, d)

  if isinstance(x, pd.DataFrame) or isinstance(x, pd.Series):
    # It's a pandas dataframe.  So we need to see if any of the dtypes are
    # categorical or object, and if so, we need to convert them.  First see if
    # we can take a shortcut without copying.
    dtype_array = x.dtypes.values if len(x.dtypes) > 0 else [x.dtypes]
    if not any(isinstance(t, CategoricalDtype)
        for t in dtype_array) and \
       not np.dtype(object) in dtype_array and \
       not np.dtype(str) in dtype_array and \
       not np.dtype(unicode) in dtype_array:
        # We can just return the matrix as-is; it's all numeric.
        t = to_matrix(x, dtype=dtype, copy=copy)
        if len(x.shape) < 2:
          d = np.zeros(1, dtype=bool)
        else:
          d = np.zeros([x.shape[1]], dtype=bool)
        return (t[0], t[1], d)

    if np.dtype(str) in dtype_array or np.dtype(unicode) in dtype_array:
      raise TypeError('cannot convert matrices with string types')

    if np.dtype(buffer) in dtype_array:
      raise TypeError("'buffer' dtype not supported")

    # If we get to here, then we are going to need to do some type conversion,
    # so go ahead and copy the dataframe and we'll work with y to make
    # modifications.
    y = x
    d = np.zeros([x.shape[1]], dtype=bool)

    # Convert any 'object', 'str', or 'unicode' types to categorical.
    convertColumns = x.select_dtypes(['object'])
    if not convertColumns.empty:
      y[convertColumns] = y[convertColumns].astype('category')

    catColumns = x.select_dtypes(['category']).columns
    if len(catColumns) > 0:
      # Do actual conversion to numeric types.  This converts to an int type.
      y = x # Copy it... not great...

      # Note that this will map NaNs (missing values or unknown categories) to
      # -1, so we will have to convert those back to NaN.
      y[catColumns] = y[catColumns].apply(
          lambda c: c.cat.codes).astype('double')
      y[catColumns].replace(to_replace=[-1], value=float('NaN'))

      # Construct dataset information: 1s represent categorical data, 0s
      # represent otherwise.
      catColumnIndices = [y.columns.get_loc(i) for i in catColumns]
      d[catColumnIndices] = 1

    # We'll have to force the second part of the tuple (whether or not to take
    # ownership) to true.
    t = to_matrix(y.apply(pd.to_numeric), dtype=dtype)
    return (t[0], True, d)

  if isinstance(x, list):
    # Get the number of dimensions.
    dims = 0
    if isinstance(x[0], list):
      dims = len(x[0])
    else:
      dims = len(x)

    d = np.zeros([dims])
    if np.lib.NumpyVersion(np.__version__) >= '2.0.0b1':
      out = np.array(x, dtype=dtype, copy=(True if copy else None))
    else:
      out = np.array(x, dtype=dtype, copy=copy)

    # Since we don't have a great way to check if these are using the same
    # memory location, we will probe manually (ugh).
    oldval = x[0][0]
    x[0][0] *= 2
    alias = False
    if out.flat[0] == x[0][0]:
      alias = True
    x[0][0] = oldval

    return (out, not alias, d)

  # If we got here, the type is not known.
  raise TypeError("given matrix is not a numpy ndarray or pandas DataFrame or "\
      "Python array; not supported at this time");



#!/usr/bin/env python
"""
test_python_binding.py

Test that passing types to Python bindings works successfully.

mlpack is free software; you may redistribute it and/or modify it under the
terms of the 3-clause BSD license.  You should have received a copy of the
3-clause BSD license along with mlpack.  If not, see
http://www.opensource.org/licenses/BSD-3-Clause for more information.
"""
import unittest
import pandas as pd
import numpy as np
import copy

from mlpack.test_python_binding import test_python_binding

class TestPythonBinding(unittest.TestCase):
  """
  This class tests the basic functionality of the Python bindings.
  """

  def testRunBindingCorrectly(self):
    """
    Test that when we run the binding correctly (with correct input parameters),
    we get the expected output.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 flag1=True)

    self.assertEqual(output['string_out'], 'hello2')
    self.assertEqual(output['int_out'], 13)
    self.assertEqual(output['double_out'], 5.0)

  def testRunBindingNoFlag(self):
    """
    If we forget the mandatory flag, we should get wrong results.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0])

    self.assertNotEqual(output['string_out'], 'hello2')
    self.assertNotEqual(output['int_out'], 13)
    self.assertNotEqual(output['double_out'], 5.0)

  def testRunBindingWrongString(self):
    """
    If we give the wrong string, we should get wrong results.
    """
    output = test_python_binding(string_in='goodbye',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 flag1=True)

    self.assertNotEqual(output['string_out'], 'hello2')

  def testRunBindingWrongInt(self):
    """
    If we give the wrong int, we should get wrong results.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=15,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 flag1=True)

    self.assertNotEqual(output['int_out'], 13)

  def testRunBindingWrongDouble(self):
    """
    If we give the wrong double, we should get wrong results.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=2.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 flag1=True)

    self.assertNotEqual(output['double_out'], 5.0)

  def testRunBadFlag(self):
    """
    If we give the second flag, this should fail.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 flag1=True,
                                 flag2=True)

    self.assertNotEqual(output['string_out'], 'hello2')
    self.assertNotEqual(output['int_out'], 13)
    self.assertNotEqual(output['double_out'], 5.0)

  def testNumpyMatrix(self):
    """
    The matrix we pass in, we should get back with the third dimension doubled
    and the fifth forgotten.
    """
    x = np.random.rand(100, 5);
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=z)

    self.assertEqual(output['matrix_out'].shape[0], 100)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x[j, i], output['matrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x[j, 2], output['matrix_out'][j, 2])

  def testNumpyMatrixForceCopy(self):
    """
    The matrix we pass in, we should get back with the third dimension doubled
    and the fifth forgotten.
    """
    x = np.random.rand(100, 5);

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_out'].shape[0], 100)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x[j, i], output['matrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x[j, 2], output['matrix_out'][j, 2])


  def testNumpyFContiguousMatrix(self):
    """
    The matrix with F_CONTIGUOUS set we pass in, we should get back with the third
    dimension doubled and the fifth forgotten.
    """
    x = np.array(np.random.rand(100, 5), order='F');
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=z)

    self.assertEqual(output['matrix_out'].shape[0], 100)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x[j, i], output['matrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x[j, 2], output['matrix_out'][j, 2])

  def testNumpyFContiguousMatrixForceCopy(self):
    """
    The matrix with F_CONTIGUOUS set we pass in, we should get back with the third
    dimension doubled and the fifth forgotten.
    """
    x = np.array(np.random.rand(100, 5), order='F');

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_out'].shape[0], 100)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x[j, i], output['matrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x[j, 2], output['matrix_out'][j, 2])

  def testPandasSeriesMatrix(self):
    """
    Test that we can pass pandas.Series as input parameter.
    """
    x = pd.Series(np.random.rand(100))
    z = x.copy(deep=True)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 smatrix_in=z)

    self.assertEqual(output['smatrix_out'].shape[0], 100)
    self.assertEqual(output['smatrix_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['smatrix_out'][i,0], x.iloc[i] * 2)


  def testPandasSeriesMatrixForceCopy(self):
    """
    Test that we can pass pandas.Series as input parameter.
    """
    x = pd.Series(np.random.rand(100))

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 smatrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['smatrix_out'].shape[0], 100)
    self.assertEqual(output['smatrix_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['smatrix_out'][i,0], x.iloc[i] * 2)

  def testPandasSeriesUMatrix(self):
    """
    Test that we can pass pandas.Series as input parameter.
    """
    x = pd.Series(np.random.randint(0, high=500, size=100))
    z = x.copy(deep=True)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 s_umatrix_in=z)

    self.assertEqual(output['s_umatrix_out'].shape[0], 100)
    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['s_umatrix_out'][i, 0], x.iloc[i] * 2)


  def testPandasSeriesUMatrixForceCopy(self):
    """
    Test that we can pass pandas.Series as input parameter.
    """
    x = pd.Series(np.random.randint(0, high=500, size=100))

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 s_umatrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['s_umatrix_out'].shape[0], 100)
    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['s_umatrix_out'][i, 0], x.iloc[i] * 2)

  def testPandasSeries(self):
    """
    Test a Pandas Series input paramter
    """
    x = pd.Series(np.random.rand(100))
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 col_in=z)

    self.assertEqual(output['col_out'].shape[0], 100)
    self.assertEqual(output['col_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['col_out'][i], z[i] * 2)

  def testPandasSeriesForceCopy(self):
    """
    Test a Pandas Series input paramter
    """
    x = pd.Series(np.random.rand(100))

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 col_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['col_out'].shape[0], 100)
    self.assertEqual(output['col_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['col_out'][i], x[i] * 2)

  def testPandasDataFrameMatrix(self):
    """
    The matrix we pass in, we should get back with the third dimension doubled
    and the fifth forgotten.
    """
    x = pd.DataFrame(np.random.rand(100, 5))
    z = x.copy(deep=True)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=z)

    self.assertEqual(output['matrix_out'].shape[0], 100)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x.iloc[j, i], output['matrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x.iloc[j, 2], output['matrix_out'][j, 2])

  def testPandasDataFrameMatrixForceCopy(self):
    """
    The matrix we pass in, we should get back with the third dimension doubled
    and the fifth forgotten.
    """
    x = pd.DataFrame(np.random.rand(100, 5))

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_out'].shape[0], 100)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x.iloc[j, i], output['matrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x.iloc[j, 2], output['matrix_out'][j, 2])

  def testArraylikeMatrix(self):
    """
    Test that we can pass an arraylike matrix.
    """
    x = [[1, 2, 3, 4, 5],
         [6, 7, 8, 9, 10],
         [11, 12, 13, 14, 15]]

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=x)

    self.assertEqual(output['matrix_out'].shape[0], 3)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    self.assertEqual(output['matrix_out'][0, 0], 1)
    self.assertEqual(output['matrix_out'][0, 1], 2)
    self.assertEqual(output['matrix_out'][0, 2], 6)
    self.assertEqual(output['matrix_out'][0, 3], 4)
    self.assertEqual(output['matrix_out'][1, 0], 6)
    self.assertEqual(output['matrix_out'][1, 1], 7)
    self.assertEqual(output['matrix_out'][1, 2], 16)
    self.assertEqual(output['matrix_out'][1, 3], 9)
    self.assertEqual(output['matrix_out'][2, 0], 11)
    self.assertEqual(output['matrix_out'][2, 1], 12)
    self.assertEqual(output['matrix_out'][2, 2], 26)
    self.assertEqual(output['matrix_out'][2, 3], 14)

  def testArraylikeMatrixForceCopy(self):
    """
    Test that we can pass an arraylike matrix.
    """
    x = [[1, 2, 3, 4, 5],
         [6, 7, 8, 9, 10],
         [11, 12, 13, 14, 15]]

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_out'].shape[0], 3)
    self.assertEqual(output['matrix_out'].shape[1], 4)
    self.assertEqual(len(x), 3)
    self.assertEqual(len(x[0]), 5)
    self.assertEqual(output['matrix_out'].dtype, np.double)
    self.assertEqual(output['matrix_out'][0, 0], 1)
    self.assertEqual(output['matrix_out'][0, 1], 2)
    self.assertEqual(output['matrix_out'][0, 2], 6)
    self.assertEqual(output['matrix_out'][0, 3], 4)
    self.assertEqual(output['matrix_out'][1, 0], 6)
    self.assertEqual(output['matrix_out'][1, 1], 7)
    self.assertEqual(output['matrix_out'][1, 2], 16)
    self.assertEqual(output['matrix_out'][1, 3], 9)
    self.assertEqual(output['matrix_out'][2, 0], 11)
    self.assertEqual(output['matrix_out'][2, 1], 12)
    self.assertEqual(output['matrix_out'][2, 2], 26)
    self.assertEqual(output['matrix_out'][2, 3], 14)

  def testNumpyUmatrix(self):
    """
    Same as testNumpyMatrix() but with an unsigned matrix.
    """
    x = np.random.randint(0, high=500, size=[100, 5])
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 umatrix_in=z)

    self.assertEqual(output['umatrix_out'].shape[0], 100)
    self.assertEqual(output['umatrix_out'].shape[1], 4)
    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x[j, i], output['umatrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x[j, 2], output['umatrix_out'][j, 2])

  def testNumpyUmatrixForceCopy(self):
    """
    Same as testNumpyMatrix() but with an unsigned matrix.
    """
    x = np.random.randint(0, high=500, size=[100, 5])

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 umatrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['umatrix_out'].shape[0], 100)
    self.assertEqual(output['umatrix_out'].shape[1], 4)
    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
    for i in [0, 1, 3]:
      for j in range(100):
        self.assertEqual(x[j, i], output['umatrix_out'][j, i])

    for j in range(100):
      self.assertEqual(2 * x[j, 2], output['umatrix_out'][j, 2])

  def testArraylikeUmatrix(self):
    """
    Test that we can pass an arraylike unsigned matrix.
    """
    x = [[1, 2, 3, 4, 5],
         [6, 7, 8, 9, 10],
         [11, 12, 13, 14, 15]]

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 umatrix_in=x)

    self.assertEqual(output['umatrix_out'].shape[0], 3)
    self.assertEqual(output['umatrix_out'].shape[1], 4)
    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
    self.assertEqual(output['umatrix_out'][0, 0], 1)
    self.assertEqual(output['umatrix_out'][0, 1], 2)
    self.assertEqual(output['umatrix_out'][0, 2], 6)
    self.assertEqual(output['umatrix_out'][0, 3], 4)
    self.assertEqual(output['umatrix_out'][1, 0], 6)
    self.assertEqual(output['umatrix_out'][1, 1], 7)
    self.assertEqual(output['umatrix_out'][1, 2], 16)
    self.assertEqual(output['umatrix_out'][1, 3], 9)
    self.assertEqual(output['umatrix_out'][2, 0], 11)
    self.assertEqual(output['umatrix_out'][2, 1], 12)
    self.assertEqual(output['umatrix_out'][2, 2], 26)
    self.assertEqual(output['umatrix_out'][2, 3], 14)

  def testArraylikeUmatrixForceCopy(self):
    """
    Test that we can pass an arraylike unsigned matrix.
    """
    x = [[1, 2, 3, 4, 5],
         [6, 7, 8, 9, 10],
         [11, 12, 13, 14, 15]]

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 umatrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['umatrix_out'].shape[0], 3)
    self.assertEqual(output['umatrix_out'].shape[1], 4)
    self.assertEqual(len(x), 3)
    self.assertEqual(len(x[0]), 5)
    self.assertEqual(output['umatrix_out'].dtype, np.dtype('intp'))
    self.assertEqual(output['umatrix_out'][0, 0], 1)
    self.assertEqual(output['umatrix_out'][0, 1], 2)
    self.assertEqual(output['umatrix_out'][0, 2], 6)
    self.assertEqual(output['umatrix_out'][0, 3], 4)
    self.assertEqual(output['umatrix_out'][1, 0], 6)
    self.assertEqual(output['umatrix_out'][1, 1], 7)
    self.assertEqual(output['umatrix_out'][1, 2], 16)
    self.assertEqual(output['umatrix_out'][1, 3], 9)
    self.assertEqual(output['umatrix_out'][2, 0], 11)
    self.assertEqual(output['umatrix_out'][2, 1], 12)
    self.assertEqual(output['umatrix_out'][2, 2], 26)
    self.assertEqual(output['umatrix_out'][2, 3], 14)

  def testTransMatrix(self):
    """
    Test that we can correctly pass a matrix in that's specified with the
    PARAM_TMATRIX_IN() macro, and it is correctly transposed.
    """
    x = np.random.rand(20, 10)
    test_python_binding(string_in='hello',
                        int_in=12,
                        double_in=4.0,
                        mat_req_in=[[1.0]],
                        col_req_in=[1.0],
                        matrix_in=x,
                        tmatrix_in=x)

  def testTransMatrixForceCopy(self):
    """
    The same test as above, but we force copies.
    """
    x = np.random.rand(20, 10)
    xt = copy.deepcopy(np.transpose(x))
    test_python_binding(string_in='hello',
                        int_in=12,
                        double_in=4.0,
                        mat_req_in=[[1.0]],
                        col_req_in=[1.0],
                        matrix_in=x,
                        tmatrix_in=x,
                        copy_all_inputs=True)

  def testCol(self):
    """
    Test a column vector input parameter.
    """
    x = np.random.rand(100)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 col_in=z)

    self.assertEqual(output['col_out'].shape[0], 100)
    self.assertEqual(output['col_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['col_out'][i], x[i] * 2)

  def testColForceCopy(self):
    """
    Test a column vector input parameter.
    """
    x = np.random.rand(100)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 col_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['col_out'].shape[0], 100)
    self.assertEqual(output['col_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['col_out'][i], x[i] * 2)

  def testUcol(self):
    """
    Test an unsigned column vector input parameter.
    """
    x = np.random.randint(0, high=500, size=100)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 ucol_in=z)

    self.assertEqual(output['ucol_out'].shape[0], 100)
    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
    for i in range(100):
      self.assertEqual(output['ucol_out'][i], x[i] * 2)

  def testUcolForceCopy(self):
    """
    Test an unsigned column vector input parameter.
    """
    x = np.random.randint(0, high=500, size=100)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 ucol_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['ucol_out'].shape[0], 100)
    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
    for i in range(100):
      self.assertEqual(output['ucol_out'][i], x[i] * 2)

  def testRow(self):
    """
    Test a row vector input parameter.
    """
    x = np.random.rand(100)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 row_in=z)

    self.assertEqual(output['row_out'].shape[0], 100)
    self.assertEqual(output['row_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['row_out'][i], x[i] * 2)

  def testRowForceCopy(self):
    """
    Test a row vector input parameter.
    """
    x = np.random.rand(100)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 row_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['row_out'].shape[0], 100)
    self.assertEqual(output['row_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['row_out'][i], x[i] * 2)

  def testUrow(self):
    """
    Test an unsigned row vector input parameter.
    """
    x = np.random.randint(0, high=500, size=100)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 urow_in=z)

    self.assertEqual(output['urow_out'].shape[0], 100)
    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['urow_out'][i], x[i] * 2)

  def testUrowForceCopy(self):
    """
    Test an unsigned row vector input parameter.
    """
    x = np.random.randint(0, high=500, size=100)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 urow_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['urow_out'].shape[0], 100)
    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['urow_out'][i], x[i] * 2)

  def testMatrixAndInfoNumpy(self):
    """
    Test that we can pass a matrix with all numeric features.
    """
    x = np.random.rand(100, 10)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_and_info_in=z)

    self.assertEqual(output['matrix_and_info_out'].shape[0], 100)
    self.assertEqual(output['matrix_and_info_out'].shape[1], 10)

    for i in range(10):
      for j in range(100):
        self.assertEqual(output['matrix_and_info_out'][j, i], x[j, i] * 2.0)

  def testMatrixAndInfoNumpyForceCopy(self):
    """
    Test that we can pass a matrix with all numeric features.
    """
    x = np.random.rand(100, 10)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_and_info_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_and_info_out'].shape[0], 100)
    self.assertEqual(output['matrix_and_info_out'].shape[1], 10)

    for i in range(10):
      for j in range(100):
        self.assertEqual(output['matrix_and_info_out'][j, i], x[j, i] * 2.0)

  def testMatrixAndInfoPandas(self):
    """
    Test that we can pass a matrix with some categorical features.
    """
    x = pd.DataFrame(np.random.rand(10, 4), columns=list('abcd'))
    x['e'] = pd.Series(['a', 'b', 'c', 'd', 'a', 'b', 'e', 'c', 'a', 'b'],
        dtype='category')
    z = x.copy(deep=True)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_and_info_in=z)

    self.assertEqual(output['matrix_and_info_out'].shape[0], 10)
    self.assertEqual(output['matrix_and_info_out'].shape[1], 5)

    cols = list('abcde')

    for i in range(4):
      for j in range(10):
        self.assertEqual(output['matrix_and_info_out'][j, i], z[cols[i]][j] * 2)

    for j in range(10):
      self.assertEqual(output['matrix_and_info_out'][j, 4], z[cols[4]][j])

  def testMatrixAndInfoPandasForceCopy(self):
    """
    Test that we can pass a matrix with some categorical features.
    """
    x = pd.DataFrame(np.random.rand(10, 4), columns=list('abcd'))
    x['e'] = pd.Series(['a', 'b', 'c', 'd', 'a', 'b', 'e', 'c', 'a', 'b'],
        dtype='category')

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_and_info_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_and_info_out'].shape[0], 10)
    self.assertEqual(output['matrix_and_info_out'].shape[1], 5)

    cols = list('abcde')

    for i in range(4):
      for j in range(10):
        self.assertEqual(output['matrix_and_info_out'][j, i], x[cols[i]][j] * 2)

    for j in range(10):
      self.assertEqual(output['matrix_and_info_out'][j, 4], x[cols[4]][j])

  def testIntVector(self):
    """
    Test that we can pass a vector of ints and get back that same vector but
    with the last element removed.
    """
    x = [1, 2, 3, 4, 5]

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 vector_in=x)

    self.assertEqual(output['vector_out'], [1, 2, 3, 4])

  def testStringVector(self):
    """
    Test that we can pass a vector of strings and get back that same vector but
    with the last element removed.
    """
    x = ['one', 'two', 'three', 'four', 'five']

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 str_vector_in=x)

    self.assertEqual(output['str_vector_out'],
        ['one', 'two', 'three', 'four'])

  def testModel(self):
    """
    First create a GaussianKernel object, then send it back and make sure we get
    the right double value.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 build_model=True)

    output2 = test_python_binding(string_in='hello',
                                  int_in=12,
                                  double_in=4.0,
                                  mat_req_in=[[1.0]],
                                  col_req_in=[1.0],
                                  model_in=output['model_out'])

    self.assertEqual(output2['model_bw_out'], 20.0)

  def testOneDimensionNumpyMatrix(self):
    """
    Test that we can pass one dimension matrix from matrix_in
    """
    x = np.random.rand(100)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 smatrix_in=z)

    self.assertEqual(output['smatrix_out'].shape[0], 100)
    self.assertEqual(output['smatrix_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['smatrix_out'][i, 0], x[i] * 2)


  def testOneDimensionNumpymatrixForceCopy(self):
    """
    Test that we can pass one dimension matrix from matrix_in
    """
    x = np.random.rand(100)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 smatrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['smatrix_out'].shape[0], 100)
    self.assertEqual(output['smatrix_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['smatrix_out'][i, 0], x[i] * 2)

  def testOneDimensionNumpyUmatrix(self):
    """
    Same as testNumpyMatrix() but with an unsigned matrix and One Dimension Matrix.
    """
    x = np.random.randint(0, high=500, size=100)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 s_umatrix_in=z)

    self.assertEqual(output['s_umatrix_out'].shape[0], 100)
    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['s_umatrix_out'][i, 0], x[i] * 2)

  def testOneDimensionNumpyUmatrixForceCopy(self):
    """
    Same as testNumpyMatrix() but with an unsigned matrix and One Dimension Matrix.
    """
    x = np.random.randint(0, high=500, size=100)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 s_umatrix_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['s_umatrix_out'].shape[0], 100)
    self.assertEqual(output['s_umatrix_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['s_umatrix_out'][i, 0], x[i] * 2)

  def testTwoDimensionCol(self):
    """
    Test that we pass Two Dimension column vetor as input paramter
    """
    x = np.random.rand(100,1)
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 col_in=z)

    self.assertEqual(output['col_out'].shape[0], 100)
    self.assertEqual(output['col_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['col_out'][i], x[i] * 2)

  def testTwoDimensionColForceCopy(self):
    """
    Test that we pass Two Dimension column vetor as input paramter
    """
    x = np.random.rand(100,1)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 col_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['col_out'].shape[0], 100)
    self.assertEqual(output['col_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['col_out'][i], x[i] * 2)

  def testTwoDimensionUcol(self):
    """
    Test that we pass Two Dimension unsigned column vector input parameter.
    """
    x = np.random.randint(0, high=500, size=[100, 1])
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 ucol_in=z)

    self.assertEqual(output['ucol_out'].shape[0], 100)
    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
    for i in range(100):
      self.assertEqual(output['ucol_out'][i], x[i] * 2)

  def testTwoDimensionUcolForceCopy(self):
    """
    Test that we pass Two Dimension unsigned column vector input parameter.
    """
    x = np.random.randint(0, high=500, size=[100, 1])

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 ucol_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['ucol_out'].shape[0], 100)
    self.assertEqual(output['ucol_out'].dtype, np.dtype('intp'))
    for i in range(100):
      self.assertEqual(output['ucol_out'][i], x[i] * 2)

  def testTwoDimensionRow(self):
    """
    Test a two dimensional row vector input parameter.
    """
    x = np.random.rand(100,1)
    z =copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 row_in=x)

    self.assertEqual(output['row_out'].shape[0], 100)
    self.assertEqual(output['row_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['row_out'][i], z[i] * 2)

  def testTwoDimensionRowForceCopy(self):
    """
    Test a two dimensional row vector input parameter.
    """
    x = np.random.rand(100,1)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 row_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['row_out'].shape[0], 100)
    self.assertEqual(output['row_out'].dtype, np.double)

    for i in range(100):
      self.assertEqual(output['row_out'][i], x[i] * 2)

  def testTwoDimensionUrow(self):
    """
    Test an unsigned two dimensional row vector input parameter.
    """
    x = np.random.randint(0, high=500, size=[100, 1])
    z = copy.deepcopy(x)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 urow_in=z)

    self.assertEqual(output['urow_out'].shape[0], 100)
    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))

    for i in range(100):
      self.assertEqual(output['urow_out'][i], x[i] * 2)

  def testTwoDimensionUrowForceCopy(self):
    """
    Test an unsigned two dimensional row vector input parameter.
    """
    x = np.random.randint(5, high=500, size=[1, 101])

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 urow_in=x,
                                 copy_all_inputs=True)

    self.assertEqual(output['urow_out'].shape[0], 101)
    self.assertEqual(output['urow_out'].dtype, np.dtype('intp'))

    for i in range(101):
      self.assertEqual(output['urow_out'][i], x[0][i] * 2)

  def testOneDimensionMatrixAndInfoPandas(self):
    """
    Test that we can pass a one dimension matrix with some categorical features.
    """
    x = pd.DataFrame(np.random.rand(10))
    z = x.copy(deep=True)

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_and_info_in=z[0])

    self.assertEqual(output['matrix_and_info_out'].shape[0], 10)

    for i in range(10):
      self.assertEqual(output['matrix_and_info_out'][i, 0], x[0][i] * 2)

  def testOneDimensionMatrixAndInfoPandasForceCopy(self):
    """
    Test that we can pass a one dimension matrix with some categorical features.
    """
    x = pd.DataFrame(np.random.rand(10))

    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 matrix_and_info_in=x[0],
                                 copy_all_inputs=True)

    self.assertEqual(output['matrix_and_info_out'].shape[0], 10)

    for j in range(10):
      self.assertEqual(output['matrix_and_info_out'][j, 0], x[0][j]*2)

  def testThrownException(self):

    """
    Test that we pass wrong type and get back TypeError
    """
    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in=10,
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=10.0,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in='bad',
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   flag2=10))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   matrix_in= 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   matrix_in= 1))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   matrix_and_info_in = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   copy_all_inputs = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   col_in = 10))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   row_in = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   str_vector_in = 'bad'))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   urow_in = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   ucol_in = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   umatrix_in = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   verbose = 10))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   flag1=True,
                                                   vector_in = 10.0))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=False,
                                                   col_req_in=[1.0],
                                                   flag1=True))

    self.assertRaises(TypeError,
                      lambda : test_python_binding(string_in='hello',
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=False,
                                                   flag1=True))

  def testModelForceCopy(self):
    """
    First create a GaussianKernel object, then send it back and make sure we get
    the right double value.
    """
    output = test_python_binding(string_in='hello',
                                 int_in=12,
                                 double_in=4.0,
                                 mat_req_in=[[1.0]],
                                 col_req_in=[1.0],
                                 build_model=True)

    output2 = test_python_binding(string_in='hello',
                                  int_in=12,
                                  double_in=4.0,
                                  mat_req_in=[[1.0]],
                                  col_req_in=[1.0],
                                  model_in=output['model_out'],
                                  copy_all_inputs=True)

    output3 = test_python_binding(string_in='hello',
                                  int_in=12,
                                  double_in=4.0,
                                  mat_req_in=[[1.0]],
                                  col_req_in=[1.0],
                                  model_in=output['model_out'])

    self.assertEqual(output2['model_bw_out'], 20.0)
    self.assertEqual(output3['model_bw_out'], 20.0)

  def testCheckInputMatricesNaN(self):
    """
    Checks that an exception is thrown if the input matrix contains
    NaN values.
    """
    x = np.random.rand(100, 5)
    a = np.random.randint(low=0, high=100)
    b = np.random.randint(low=0, high=5)
    x[a][b] = np.nan
    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   matrix_in=x,
                                                   check_input_matrices=True))

    x_vec = np.random.rand(100)
    a = np.random.randint(low=0, high=100)
    x_vec[a] = np.nan
    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   row_in=x_vec,
                                                   check_input_matrices=True))

    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   col_in=x_vec,
                                                   check_input_matrices=True))

    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   matrix_and_info_in=x,
                                                   check_input_matrices=True))

  def testCheckInputMatricesInf(self):
    """
    Checks that an exception is thrown if the input matrix contains
    inf values.
    """
    x = np.random.rand(100, 5)
    a = np.random.randint(low=0, high=100)
    b = np.random.randint(low=0, high=5)
    x[a][b] = np.inf
    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   matrix_in=x,
                                                   check_input_matrices=True))

    x_vec = np.random.rand(100)
    a = np.random.randint(low=0, high=100)
    x_vec[a] = np.inf
    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   row_in=x_vec,
                                                   check_input_matrices=True))

    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   col_in=x_vec,
                                                   check_input_matrices=True))

    self.assertRaises(RuntimeError,
                      lambda : test_python_binding(string_in="hello",
                                                   int_in=12,
                                                   double_in=4.0,
                                                   mat_req_in=[[1.0]],
                                                   col_req_in=[1.0],
                                                   matrix_and_info_in=x,
                                                   check_input_matrices=True))

if __name__ == '__main__':
  unittest.main()
